window.Gleam.initSearch([{"doc":"glacier","title":"glacier","content":"# Glacier · Gleam Incremental Interactive Unit Testing\n\n![Under construction](./resources/glacier-logo.png)\n\n[![Package Version](https://img.shields.io/hexpm/v/glacier)](https://hex.pm/packages/glacier)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glacier/)\n\n## Introduction\n\n**Glacier** brings incremental interactive unit testing to [Gleam](https://gleam.run).\nIt is meant as a drop-in replacement for [Gleeunit](https://hexdocs.pm/gleeunit) and it relies on it, internally.\n\n*Glacier* differs from *Gleeunit* insofar, that it let&#39;s you:\n\n1. Pass in an interactive flag `gleam test -- --glacier`; save a module and only related tests will rerun.\n2. Pass in a specific unit test modules to rerun `gleam test -- test/my_module_test.gleam`.\n3. If `gleam test` is passed without any `--`-arguments it behaves the same as *Gleeunit*.\n4. You can still pass in `--target erlang` or `--target javascript`, like so: `gleam test --target erlang -- --glacier` or like `gleam test --target javascript -- test/my_module_test.gleam`.\n\nTo enable this behavior, all you have to do is add *Glacier* as a dev dependency, aka `gleam add glacier`, open `./test/YOUR_PROJECT.gleam` and replace `gleeunit.main()` with `glacier.main()`.\n\n## How does it work?\n\n1. `gleam test` passes through `glacier.run()` and simply executes `gleeunit.main()` as if *Gleeunit* was used directly.\n2. `gleam test -- test_module_a test_module_b` passes through `glacier.run()` and executes `gleeunit.test_modules(modules_list)` where `modules_list` is `[&quot;foo&quot;, &quot;bar&quot;]`. The given modules are checked if they exist as either `.gleam` or `.erl` test module files and then *Gleeunit* runs these test modules.\n3. `gleam test -- --glacier` enters `glacier.run()` and starts a file watcher: Upon changes in module files in `./test` it just passes those through as `gleam test -- changed_test_module`(so re-saving test files executes the single test), and if a module file in `./src` got changed it parses that changed module file for any imported modules and puts the module and all chained imported modules in a distinct list of modules that should be tested. Then all test module files are read and imports of those are gathered one by one and cross matched against that list. The result is a list of test modules that need to be run, which then gets done by executing a shell call similar to `gleam test -- detected_test_module_a detected_test_module_b detected_test_module_c etc`, aka jumps to `2.`.\n\n## TODO\n\n- Erlang: Introduce some delay between the file watcher picking up a change and the test running, so that if you find-replace-save-all via an editor it does not try to run the tests 10 times. This requires some medium large changes as the code to detect imports and dependent imports needs to run for 1..n modules and if test modules are affected it needs to be handled separately (added afterwards distinct, unique).\n- Handling of JavaScript src and test modules for DENO: &lt;https://deno.land/api@v1.29.1?s=Deno.watchFs&gt;.\n- Save import lists per module into an in-memory hash-table. Before a file gets parsed for its imports, build the hash and check if a cached version already exists.\n- Use set instead of lists in some places?\n- Once gleam does only recompile changed modules: Do not test dependencies if they had been tested recently, aka the first time a module it saved all its dependencies are tested but afterwards only if they changed by keeping a cache table with the module name and an mtime it last run the tests for those modules.\n- Backport gleeunit changes. Make this lib depend on the backported version of gleeunit instead of including it.\n- Handling of Elixir src and test modules.\n\n## Known Caveats\n\n- With *Glacier* you can only run interactive incremental test against a single target, Erlang or JavaScript, at a time.\n\n## Installation &amp; Usage\n\nFor Erlang this library depends on [fs](https://hexdocs.pm/fs/). Because of this on Linux you will need [`inotify-tools` to be installed](https://github.com/synrc/fs#backends). On Mac and Windows it should work out of the box.\n\nFor NodeJS version 18 LTS is supported and development always and only happens on the latest stable LTS, such as Node.js say v18.12.1.\n\nIf available on Hex this package can be added to your Gleam project:\n\n1. Run `gleam add glacier`\n2. Open `./test/YOUR_PROJECT.gleam` and replace `gleeunit.main()` with `glacier.main()`\n3. Run `gleam test`\n   - save any test module (within `./test`) file to re-run that single test\n   - save any src module (within `./src`) to run all associated tests. Associated tests are test modules where the module is imported or where any of the module&#39;s imports and their import&#39;s imports (import chain) are imported into.\n\n... and its documentation can be found at &lt;https://hexdocs.pm/glacier&gt;.\n\n## Quick start\n\n```sh\ngit clone https://github.com/inoas/glacier.git\ncd glacier\ngleam run   # Show instructions how to use the library stand alone\ngleam test  # Run the tests on this library\n```\n\n## Status\n\nAlpha quality, thus while this is still...\n\n![Under construction](https://web.archive.org/web/20090829023556im_/http://geocities.com/okitsugu/underconstruction.gif)\n\n...it should be working now.\n","url":"index.html"},{"doc":"glacier_demo/glacier_demo_module_a","title":"function_1","content":"pub fn function_1() -&gt; Int\n\nSynonyms:\nglacier_demo/glacier_demo_module_a.function_1\nglacier_demo/glacier_demo_module_a function_1","url":"glacier_demo/glacier_demo_module_a.html#function_1"},{"doc":"glacier_demo/glacier_demo_module_a","title":"function_2","content":"pub fn function_2() -&gt; Int\n\nSynonyms:\nglacier_demo/glacier_demo_module_a.function_2\nglacier_demo/glacier_demo_module_a function_2","url":"glacier_demo/glacier_demo_module_a.html#function_2"},{"doc":"glacier_demo/glacier_demo_module_a","title":"glacier_demo/glacier_demo_module_a","content":"","url":"glacier_demo/glacier_demo_module_a.html"},{"doc":"gleeunit/should","title":"be_error","content":"pub external fn be_error(Result(a, b)) -&gt; b\n\nSynonyms:\ngleeunit/should.be_error\ngleeunit/should be_error","url":"gleeunit/should.html#be_error"},{"doc":"gleeunit/should","title":"be_false","content":"pub fn be_false(actual: Bool) -&gt; Nil\n\nSynonyms:\ngleeunit/should.be_false\ngleeunit/should be_false","url":"gleeunit/should.html#be_false"},{"doc":"gleeunit/should","title":"be_ok","content":"pub external fn be_ok(Result(a, b)) -&gt; a\n\nSynonyms:\ngleeunit/should.be_ok\ngleeunit/should be_ok","url":"gleeunit/should.html#be_ok"},{"doc":"gleeunit/should","title":"be_true","content":"pub fn be_true(actual: Bool) -&gt; Nil\n\nSynonyms:\ngleeunit/should.be_true\ngleeunit/should be_true","url":"gleeunit/should.html#be_true"},{"doc":"gleeunit/should","title":"equal","content":"pub external fn equal(a, a) -&gt; Nil\n\nSynonyms:\ngleeunit/should.equal\ngleeunit/should equal","url":"gleeunit/should.html#equal"},{"doc":"gleeunit/should","title":"fail","content":"pub fn fail() -&gt; Nil\n\nSynonyms:\ngleeunit/should.fail\ngleeunit/should fail","url":"gleeunit/should.html#fail"},{"doc":"gleeunit/should","title":"not_equal","content":"pub external fn not_equal(a, a) -&gt; Nil\n\nSynonyms:\ngleeunit/should.not_equal\ngleeunit/should not_equal","url":"gleeunit/should.html#not_equal"},{"doc":"gleeunit/should","title":"gleeunit/should","content":" A module for testing your Gleam code. The functions found here are\n compatible with the Erlang eunit test framework.\n\n More information on running eunit can be found in [the rebar3\n documentation](https://rebar3.org/docs/testing/eunit/).","url":"gleeunit/should.html"},{"doc":"gleeunit","title":"main","content":"pub fn main() -&gt; Nil\n Find and run all test functions for the current project using Erlang&#39;s EUnit\n test framework.\n\n Any Erlang or Gleam function in the `test` directory with a name editing in\n `_test` is considered a test function and will be run.\n\n If running on JavaScript tests will be run with a custom test runner.\n\nSynonyms:\ngleeunit.main\ngleeunit main","url":"gleeunit.html#main"},{"doc":"gleeunit","title":"run","content":"pub fn run(for test_module_files: List(String)) -&gt; Nil\n Runs a specific list of test modules\n\nSynonyms:\ngleeunit.run\ngleeunit run","url":"gleeunit.html#run"},{"doc":"gleeunit","title":"gleeunit","content":"","url":"gleeunit.html"},{"doc":"glacier","title":"ModuleKind","content":"pub type ModuleKind {\n  SrcModuleKind\n  TestModuleKind\n}\n\nSrcModuleKind\n\n\nTestModuleKind\n\n\nSynonyms:\nglacier.ModuleKind\nglacier ModuleKind","url":"glacier.html#ModuleKind"},{"doc":"glacier","title":"shellout_lookups","content":"pub const shellout_lookups: List(\n  #(List(String), List(#(String, List(String)))),\n) = [\n  #(\n    [&quot;color&quot;, &quot;background&quot;],\n    [#(&quot;lightblue&quot;, [&quot;156&quot;, &quot;231&quot;, &quot;255&quot;])],\n  ),\n]\n\nSynonyms:\nglacier.shellout_lookups\nglacier shellout_lookups","url":"glacier.html#shellout_lookups"},{"doc":"glacier","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nglacier.main\nglacier main","url":"glacier.html#main"},{"doc":"glacier","title":"run","content":"pub fn run() -&gt; Nil\n\nSynonyms:\nglacier.run\nglacier run","url":"glacier.html#run"},{"doc":"glacier","title":"glacier","content":"","url":"glacier.html"},{"doc":"glacier_demo/glacier_demo_module_d","title":"main","content":"pub fn main() -&gt; String\n\nSynonyms:\nglacier_demo/glacier_demo_module_d.main\nglacier_demo/glacier_demo_module_d main","url":"glacier_demo/glacier_demo_module_d.html#main"},{"doc":"glacier_demo/glacier_demo_module_d","title":"glacier_demo/glacier_demo_module_d","content":"","url":"glacier_demo/glacier_demo_module_d.html"},{"doc":"glacier_demo/glacier_demo_module_c","title":"function_5","content":"pub fn function_5() -&gt; Int\n\nSynonyms:\nglacier_demo/glacier_demo_module_c.function_5\nglacier_demo/glacier_demo_module_c function_5","url":"glacier_demo/glacier_demo_module_c.html#function_5"},{"doc":"glacier_demo/glacier_demo_module_c","title":"function_6","content":"pub fn function_6() -&gt; Int\n\nSynonyms:\nglacier_demo/glacier_demo_module_c.function_6\nglacier_demo/glacier_demo_module_c function_6","url":"glacier_demo/glacier_demo_module_c.html#function_6"},{"doc":"glacier_demo/glacier_demo_module_c","title":"function_a","content":"pub fn function_a() -&gt; String\n\nSynonyms:\nglacier_demo/glacier_demo_module_c.function_a\nglacier_demo/glacier_demo_module_c function_a","url":"glacier_demo/glacier_demo_module_c.html#function_a"},{"doc":"glacier_demo/glacier_demo_module_c","title":"glacier_demo/glacier_demo_module_c","content":"","url":"glacier_demo/glacier_demo_module_c.html"},{"doc":"glacier_demo/glacier_demo_module_b","title":"a","content":"pub fn a() -&gt; String\n\nSynonyms:\nglacier_demo/glacier_demo_module_b.a\nglacier_demo/glacier_demo_module_b a","url":"glacier_demo/glacier_demo_module_b.html#a"},{"doc":"glacier_demo/glacier_demo_module_b","title":"function_3","content":"pub fn function_3() -&gt; Int\n function doc import in_fn_docblock should be ignored\nSynonyms:\nglacier_demo/glacier_demo_module_b.function_3\nglacier_demo/glacier_demo_module_b function_3","url":"glacier_demo/glacier_demo_module_b.html#function_3"},{"doc":"glacier_demo/glacier_demo_module_b","title":"function_4","content":"pub fn function_4() -&gt; Int\n\nSynonyms:\nglacier_demo/glacier_demo_module_b.function_4\nglacier_demo/glacier_demo_module_b function_4","url":"glacier_demo/glacier_demo_module_b.html#function_4"},{"doc":"glacier_demo/glacier_demo_module_b","title":"glacier_demo/glacier_demo_module_b","content":" module doc import in_module_docblock should be ignored","url":"glacier_demo/glacier_demo_module_b.html"},{"doc":"glacier_demo/glacier_demo_module_e","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nglacier_demo/glacier_demo_module_e.main\nglacier_demo/glacier_demo_module_e main","url":"glacier_demo/glacier_demo_module_e.html#main"},{"doc":"glacier_demo/glacier_demo_module_e","title":"glacier_demo/glacier_demo_module_e","content":"","url":"glacier_demo/glacier_demo_module_e.html"}]);